package com.baijiayun.download;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.text.TextUtils;
import android.text.format.Formatter;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.constant.VideoDefinition;
import com.baijiayun.download.constant.DownloadType;
import com.baijiayun.download.constant.TaskStatus;
import com.baijiayun.network.HttpException;
import com.baijiayun.playback.PlaybackActivity;
import com.baijiayun.video.NewVideoActivity;
import com.baijiayun.videoplayer.core.demo.R;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * Created by Shubo on 2017/12/11.
 */

public class SimpleVideoDownloadActivity extends AppCompatActivity {

    EditText etVideoID;
    EditText etToken;
    Button btnAdd;
    RecyclerView rvDownload;
    EditText sessionIdEt;

    private DownloadManager manager;
    private DownloadAdapter adapter;
    //默认下载点播视频
    private DownloadType downloadType = DownloadType.VIDEO;
    private Disposable videoDownloadDisposable, playbackDisposable;
    //清晰度从前往后匹配，音频>720p>超清>高清>标清>1080p
    private List<VideoDefinition> definitionList = new ArrayList<>(Arrays.asList(/*VideoDefinition.Audio,*/ VideoDefinition._720P,
            VideoDefinition.SHD, VideoDefinition.HD, VideoDefinition.SD, VideoDefinition._1080P));

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_simple_download);
        initView();
        ((RadioGroup) findViewById(R.id.download_type_rg)).setOnCheckedChangeListener((group, checkedId) -> {
            if(checkedId == R.id.rg_player){
                downloadType = DownloadType.VIDEO;
            } else{
                downloadType = DownloadType.PLAYBACK;
            }
        });

        // 初始化下载
        manager = CustomDownloadService.getDownloadManager(getApplicationContext());
        // 设置缓存文件路径
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            manager.setTargetFolder(getFilesDir() + "/bb_video_downloaded/");
        } else {
            manager.setTargetFolder(Environment.getExternalStorageDirectory().getAbsolutePath() + "/bb_video_downloaded/");
        }

        //设置下载清晰度优先匹配顺序,SDK内部匹配规则同definitionList
        manager.setPreferredDefinitionList(definitionList);
        manager.loadDownloadInfo();

        adapter = new DownloadAdapter();
        rvDownload.setLayoutManager(new LinearLayoutManager(this));
        rvDownload.setAdapter(adapter);

        btnAdd.setOnClickListener(v -> {
            String id = etVideoID.getText().toString();
            if (TextUtils.isEmpty(id)) {
                return;
            }
            String token = etToken.getText().toString();
            if (TextUtils.isEmpty(token)) {
                token = "test12345678";
            }
            String sessionId = sessionIdEt.getText().toString();
            if (TextUtils.isEmpty(sessionId)) {
                sessionId = "-1";
            }
            if(downloadType == DownloadType.VIDEO){
                newVideoDownloadTask(Long.valueOf(id), token);
            } else{
                newPlaybackDownloadTask(Long.valueOf(id), token, Long.valueOf(sessionId));
            }
        });

        //TODO 下载过程中监听网络变化
        manager.registerNetReceiver(new OnNetChangeListener() {
            @Override
            public void onWifi() {
                // do nothing
            }

            @Override
            public void onMobile() {
                Toast.makeText(SimpleVideoDownloadActivity.this, "当前使用手机流量", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onDisConnect() {
                Toast.makeText(SimpleVideoDownloadActivity.this, "网络断开，请检查网络", Toast.LENGTH_LONG).show();

            }

            @Override
            public void onNoAvailable() {
                Toast.makeText(SimpleVideoDownloadActivity.this, "当前无可用网络", Toast.LENGTH_LONG).show();
            }
        });
    }

    private void initView() {
        etVideoID = findViewById(R.id.activity_simple_download_et);
        etToken = findViewById(R.id.activity_simple_download_token);
        btnAdd = findViewById(R.id.activity_simple_download_add);
        rvDownload = findViewById(R.id.activity_simple_download_rv);
        sessionIdEt = findViewById(R.id.activity_simple_download_session_id_et);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //TODO 反注册网络监听
        manager.unregisterNetReceiver();
        RxUtils.dispose(videoDownloadDisposable);
        RxUtils.dispose(playbackDisposable);
    }

    private void newVideoDownloadTask(long videoId, String token) {
        // 点播下载
        videoDownloadDisposable = manager.newVideoDownloadTask("video%-$#e1测试=@/*?<>|*:.", videoId, token, "extraInfo")
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(downloadTask -> adapter.notifyDataSetChanged(), throwable -> {
                    throwable.printStackTrace();
                    Toast.makeText(SimpleVideoDownloadActivity.this, throwable.getMessage(), Toast.LENGTH_LONG).show();
                });
    }

    private void newPlaybackDownloadTask(long roomId, String token, long sessionId){
        playbackDisposable = manager.newPlaybackDownloadTask("playback", roomId, sessionId, token, "extraInfo")
        .observeOn(AndroidSchedulers.mainThread())
        .subscribe(downloadTask -> adapter.notifyDataSetChanged(), throwable -> {
            throwable.printStackTrace();
            Toast.makeText(SimpleVideoDownloadActivity.this, throwable.getMessage(), Toast.LENGTH_LONG).show();
        });
    }

    private class DownloadAdapter extends RecyclerView.Adapter<DownloadViewHolder> {

        @Override
        public DownloadViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
            View view = View.inflate(SimpleVideoDownloadActivity.this, R.layout.item_download_manager, null);
            return new DownloadViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final DownloadViewHolder holder, int i) {
            final DownloadTask task = manager.getAllTasks().get(i);
            final DownloadModel model = task.getVideoDownloadInfo();
            holder.videoName.setText("视频名称：" + model.videoName);
            holder.fileName.setText("文件名称：" + model.targetName);
            String downloadLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getDownloadedLength());
            String totalLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getTotalLength());
            holder.downloadSize.setText(downloadLength + "/" + totalLength);
            int progress = (int) (task.getDownloadedLength() / (float) task.getTotalLength() * 100);
            holder.pb.setProgress(progress);
            holder.tvProgress.setText(progress + "%");
            holder.netSpeed.setVisibility(task.getTaskStatus() == TaskStatus.Downloading ? View.VISIBLE : View.INVISIBLE);
            switch (task.getTaskStatus()) {
                case Error:
                    holder.download.setText("出错");
                    holder.download.setOnClickListener(v -> task.start());
                    break;
                case New:
                    holder.download.setText("下载");
                    holder.tvProgress.setText("0%");
                    holder.download.setOnClickListener(v -> task.start());
                    break;
                case Pause:
                    holder.download.setText("继续");
                    holder.download.setOnClickListener(v -> task.start());
                    break;
                case Finish:
                    holder.download.setText("完成");
                    holder.tvProgress.setText("100%");
                    holder.netSpeed.setVisibility(View.INVISIBLE);
                    showPlayButton(holder, task);
                    break;
                case Downloading:
                    holder.download.setText("暂停");
                    holder.download.setOnClickListener(v -> task.pause());
                    break;
                default:
                    break;
            }

            holder.removeFile.setOnClickListener(v -> task.deleteFiles());

            task.setDownloadListener(new DownloadListener() {
                @Override
                public void onProgress(final DownloadTask task) {
                    String downloadLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getDownloadedLength());
                    String totalLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getTotalLength());
                    holder.downloadSize.setText(downloadLength + "/" + totalLength);
                    int progress = (int) (task.getDownloadedLength() / (float) task.getTotalLength() * 100);
                    holder.pb.setProgress(progress);
                    holder.tvProgress.setText(progress + "%");
                    holder.netSpeed.setVisibility(View.VISIBLE);
                    String speed = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getSpeed());
                    holder.netSpeed.setText(speed + "/s");
                    holder.download.setText("暂停");
                    holder.download.setOnClickListener(v -> task.pause());
                }

                @Override
                public void onError(final DownloadTask task, HttpException e) {
                    holder.download.setText("出错");
                    holder.download.setOnClickListener(v -> task.start());
                    e.printStackTrace();
                    //下载地址已失效,5103(token已失效)
//                    if (e.getCode() == 403 || e.getCode() >= 5101 && e.getCode() <= 5103) {
//                        //TODO 需要用户传入新的token, 重新获取视频下载地址
//                        newDownloadVideoTask(String.valueOf(task.getDownloadInfo().videoId), "test12345678");
//                    }
                }

                @Override
                public void onPaused(final DownloadTask task) {
                    holder.download.setText("继续");
                    holder.download.setOnClickListener(v -> task.start());
                }

                @Override
                public void onStarted(final DownloadTask task) {
                    holder.download.setText("暂停");
                    holder.download.setOnClickListener(v -> task.pause());
                }

                @Override
                public void onFinish(DownloadTask task) {
                    holder.pb.setProgress(100);
                    holder.tvProgress.setText("100%");
                    holder.netSpeed.setVisibility(View.INVISIBLE);
                    String downloadLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getDownloadedLength());
                    String totalLength = Formatter.formatFileSize(SimpleVideoDownloadActivity.this, task.getTotalLength());
                    holder.downloadSize.setText(downloadLength + "/" + totalLength);
                    holder.download.setText("完成");
                    holder.download.setOnClickListener(null);
                    showPlayButton(holder, task);
                }

                @Override
                public void onDeleted(DownloadTask task) {
                    adapter.notifyDataSetChanged();
                }
            });
        }

        @Override
        public int getItemCount() {
            return manager.getAllTasks().size();
        }

        private void showPlayButton(DownloadViewHolder holder, DownloadTask task){
            holder.playBtn.setVisibility(View.VISIBLE);
            holder.playBtn.setOnClickListener(v -> {
                if(task.getDownloadType() == DownloadType.VIDEO){
                    Intent intent = new Intent(SimpleVideoDownloadActivity.this, NewVideoActivity.class);
                    intent.putExtra("videoDownloadInfo", task.getVideoDownloadInfo());
                    startActivity(intent);
                } else{
                    Intent intent = new Intent(SimpleVideoDownloadActivity.this, PlaybackActivity.class);
                    intent.putExtra("videoDownloadInfo", task.getVideoDownloadInfo());
                    intent.putExtra("signalDownloadInfo",  task.getSignalDownloadInfo());
                    startActivity(intent);
                }
            });
        }
    }

    private class DownloadViewHolder extends RecyclerView.ViewHolder {

        private TextView videoName;
        private TextView fileName;
        private TextView downloadSize;
        private TextView tvProgress;
        private TextView netSpeed;
        private Button download;
        private ProgressBar pb;
        private Button removeFile;
        private Button playBtn;

        public DownloadViewHolder(View convertView) {
            super(convertView);
            videoName = convertView.findViewById(R.id.video_name_tv);
            fileName = convertView.findViewById(R.id.file_name_tv);
            downloadSize = convertView.findViewById(R.id.downloadSize);
            tvProgress = convertView.findViewById(R.id.tvProgress);
            netSpeed = convertView.findViewById(R.id.netSpeed);
            download = convertView.findViewById(R.id.start);
            pb = convertView.findViewById(R.id.pb_progress);
            removeFile = convertView.findViewById(R.id.remove_file);
            playBtn = convertView.findViewById(R.id.play);
        }
    }
}
