package com.baijiayun.playback;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.videoplayer.core.demo.R;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;


/**
 * Created by yanglei on 2016/12/22.
 */

public class UserListFragment extends Fragment {

    private PBRoom mRoom;
    private Disposable onlineUserDisposable;

    public void setRoom(PBRoom room) {
        mRoom = room;
        onlineUserDisposable = mRoom.getOnlineUserVM().getObservableOfOnlineUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserModels -> userAdapter.notifyDataSetChanged());
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_user_list, container, false);
        RecyclerView mRecyclerView = view.findViewById(R.id.userList);
        mRecyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        mRecyclerView.setAdapter(userAdapter);
        return view;
    }

    private class ViewHolder extends RecyclerView.ViewHolder {

        private TextView userInfo;
        public ViewHolder(View itemView) {
            super(itemView);
            userInfo = itemView.findViewById(R.id.userinfo);
        }
    }

    private RecyclerView.Adapter userAdapter = new RecyclerView.Adapter<ViewHolder>() {

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.layout_item_user, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
            IUserModel userModel = mRoom.getOnlineUserVM().getUser(position);
            if(userModel != null){
                holder.userInfo.setText(String.valueOf(userModel.getName()));
            }
        }

        @Override
        public int getItemCount() {
            return mRoom == null ? 0 : mRoom.getOnlineUserVM().getUserCount();
        }
    };


    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(onlineUserDisposable);
    }
}
