package com.baijiayun.video;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import com.baijiayun.constant.VideoDefinition;
import com.baijiayun.videoplayer.IBJYVideoPlayer;
import com.baijiayun.videoplayer.core.demo.R;
import com.baijiayun.videoplayer.listeners.OnBufferingListener;
import com.baijiayun.videoplayer.player.PlayerStatus;
import com.baijiayun.videoplayer.render.AspectRatio;
import com.baijiayun.videoplayer.render.IRender;
import com.baijiayun.videoplayer.widget.BJYPlayerView;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by yangjingming on 2018/7/30.
 */

public class FirstFragment extends Fragment{

    private View firstView;

    private TextView tvPlayerStatus, tvTime, tvBuffering;

    private CheckBox cb;

    private IBJYVideoPlayer videoPlayer;

    private BJYPlayerView playerView;
    List<VideoDefinition> videoDefinitions;
    private SharedPreferences sp;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState){
        super.onCreate(savedInstanceState);
        sp = getActivity().getSharedPreferences("demo_attr_cache", Context.MODE_PRIVATE);
    }

    public void setArguments(Context context, BJYPlayerView playerView, IBJYVideoPlayer videoPlayer){
        this.playerView = playerView;
        this.videoPlayer = videoPlayer;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, Bundle saveInstanceState){
        super.onCreateView(inflater, container, saveInstanceState);
        firstView = LayoutInflater.from(getActivity()).inflate(R.layout.activity_first_fragment, null);

        initData();
        initPlayer();

        initButton();
        initDataSourceControlSection();
        initPreferredDefinitionControlSection();
        initPlayerControlSection();
        initPlayerViewControlSection();

        setupContainers(R.id.tv_data_source, R.id.rl_data_source);
        setupContainers(R.id.tv_preferred_definition, R.id.rl_definition);
        setupContainers(R.id.tv_view, R.id.rl_view);
        setupContainers(R.id.tv_player, R.id.rl_player);

        return firstView;
    }

    private void initData(){
        String vid = sp.getString("vid", null);
        String token = sp.getString("token", null);

        if (!TextUtils.isEmpty(vid)) {
            ((EditText) firstView.findViewById(R.id.et_vid)).setText(vid);
        }
        if (!TextUtils.isEmpty(token)) {
            ((EditText) firstView.findViewById(R.id.et_token)).setText(token);
        }
    }


    private void initPlayer(){
        videoPlayer.setOnTokenInvalidListener((vid, callback) -> {
            // request new token
//            String newToken = "newToken";
//            callback.onTokenFetchSuccess(newToken);
        });

        videoPlayer.addOnPlayerErrorListener(error -> {
            Log.e("PlayerError", error.getCode() + " " + error.getMessage());
            showToast("error: " + error.getCode() + " " + error.getMessage());
            TextView tv = firstView.findViewById(R.id.tv_error_log);
            String text = tv.getText().toString();
            text += "error: " + error.getCode() + " " + error.getMessage() + "\n";
            tv.setText(text);
        });

        videoPlayer.addOnPlayerStatusChangeListener(status -> {
            Log.d("yjm", "player status=" + status.name() + ", " + videoPlayer.isPlaying());
            tvPlayerStatus.setText("status: " + status.name());
            if (status == PlayerStatus.STATE_INITIALIZED) {
                ((Button) firstView.findViewById(R.id.button_definition)).setText(videoPlayer.getVideoInfo().getDefinition().name());
            }
        });
        videoPlayer.addOnBufferUpdateListener(bufferedPercentage -> tvBuffering.setText("buffered: " + bufferedPercentage + "%"));
        videoPlayer.addOnBufferingListener(new OnBufferingListener() {
            @Override
            public void onBufferingStart() {
                firstView.findViewById(R.id.tv_player_buffering).setVisibility(View.VISIBLE);
            }

            @Override
            public void onBufferingEnd() {
                firstView.findViewById(R.id.tv_player_buffering).setVisibility(View.INVISIBLE);
            }
        });
        videoPlayer.addOnPlayingTimeChangeListener((currentTime, duration) -> {
            Log.d("yjm", "player time: " + currentTime + "/" + duration);
            tvTime.setText("time: " + currentTime + "/" + duration + "  percent: " + currentTime * 100 / duration + "%");
        });
    }

    private void initButton(){
        MyOnclickListener myOnclickListener = new MyOnclickListener();
        firstView.findViewById(R.id.btn_sd).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.btn_hd).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.btn_720).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.btn_1080).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.btn_shd).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.btn_clear).setOnClickListener(myOnclickListener);

        firstView.findViewById(R.id.cb_break_point_play).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.cb_background_play).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.cb_loop_play).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.button_definition).setOnClickListener(myOnclickListener);

        firstView.findViewById(R.id.button16_9).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.button4_3).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.button_fill).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.button_fit).setOnClickListener(myOnclickListener);
        firstView.findViewById(R.id.button_origin).setOnClickListener(myOnclickListener);
    }


    private class MyOnclickListener implements View.OnClickListener{

        @Override
        public void onClick(View v) {
            switch (v.getId()) {
                case R.id.btn_sd:
                    if (!videoDefinitions.contains(VideoDefinition.SD)) {
                        videoDefinitions.add(VideoDefinition.SD);
                        notifyDefinitionChanged();
                    }
                    break;
                case R.id.btn_hd:
                    if (!videoDefinitions.contains(VideoDefinition.HD)) {
                        videoDefinitions.add(VideoDefinition.HD);
                        notifyDefinitionChanged();
                    }
                    break;
                case R.id.btn_shd:
                    if (!videoDefinitions.contains(VideoDefinition.SHD)) {
                        videoDefinitions.add(VideoDefinition.SHD);
                        notifyDefinitionChanged();
                    }
                    break;
                case R.id.btn_720:
                    if (!videoDefinitions.contains(VideoDefinition._720P)) {
                        videoDefinitions.add(VideoDefinition._720P);
                        notifyDefinitionChanged();
                    }
                    break;
                case R.id.btn_1080:
                    if (!videoDefinitions.contains(VideoDefinition._1080P)) {
                        videoDefinitions.add(VideoDefinition._1080P);
                        notifyDefinitionChanged();
                    }
                    break;
                case R.id.btn_clear:
                    videoDefinitions.clear();
                    notifyDefinitionChanged();
                    break;
                case R.id.cb_break_point_play:
                    cb = firstView.findViewById(R.id.cb_break_point_play);
                    if(cb.isChecked()){
                        videoPlayer.enableBreakPointMemory(getContext());
                    }
                    break;
                case R.id.cb_background_play:
                    cb = firstView.findViewById(R.id.cb_background_play);
                    videoPlayer.supportBackgroundAudio(cb.isChecked());
                    break;
                case R.id.cb_loop_play:
                    cb = firstView.findViewById(R.id.cb_loop_play);
                    videoPlayer.supportLooping(cb.isChecked());
                    break;
                case R.id.button_definition:
                    if (videoPlayer.getVideoInfo() == null) {
                        return;
                    }
                    final List<VideoDefinition> definitions = videoPlayer.getVideoInfo().getSupportedDefinitionList();
                    if (definitions == null) {
                        return;
                    }
                    List<String> src = new ArrayList<>(definitions.size());
                    for (VideoDefinition definition : definitions) {
                        src.add(definition.name());
                    }
                    //showDefinitions
                    new AlertDialog.Builder(getActivity())
                            .setTitle("清晰度")
                            .setItems(src.toArray(new String[]{}), (dialogInterface, i) -> videoPlayer.changeDefinition(definitions.get(i))).show();
                    break;
                case R.id.button4_3:
                    playerView.setAspectRatio(AspectRatio.AspectRatio_4_3);
                    break;
                case R.id.button16_9:
                    playerView.setAspectRatio(AspectRatio.AspectRatio_16_9);
                    break;
                case R.id.button_fill:
                    playerView.setAspectRatio(AspectRatio.AspectRatio_FILL_PARENT);
                    break;
                case R.id.button_fit:
                    playerView.setAspectRatio(AspectRatio.AspectRatio_FIT_PARENT);
                    break;
                case R.id.button_origin:
                    playerView.setAspectRatio(AspectRatio.AspectRatio_ORIGIN);
                    break;
                default:
                    break;

            }
        }
    }

    private void setupContainers(final int tv_id, final int rl_id) {
        firstView.findViewById(tv_id).setOnClickListener(v -> {
            View view = firstView.findViewById(rl_id);
            if (view.getVisibility() == View.VISIBLE) {
                view.setVisibility(View.GONE);
            } else {
                view.setVisibility(View.VISIBLE);
            }
            setupContainerLabel(tv_id, view.getVisibility() == View.VISIBLE);
        });
        setupContainerLabel(tv_id, firstView.findViewById(rl_id).getVisibility() == View.VISIBLE);
    }


    private void setupContainerLabel(int tv_id, boolean isOpen) {
        TextView tv = firstView.findViewById(tv_id);
        String text = tv.getText().toString();
        text = text.substring(0, text.length() - 1);
        text += isOpen ? "\u25B2" : "\u25BC";
        tv.setText(text);
    }

    private void initPlayerViewControlSection() {
        firstView.findViewById(R.id.button2).setOnClickListener(v -> playerView.setRenderType(IRender.RENDER_TYPE_SURFACE_VIEW));
        firstView.findViewById(R.id.button3).setOnClickListener(v -> playerView.setRenderType(IRender.RENDER_TYPE_TEXTURE_VIEW));
    }
    private void initPlayerControlSection() {
        tvPlayerStatus = firstView.findViewById(R.id.tv_player_status);
        tvTime = firstView.findViewById(R.id.tv_player_time);
        tvBuffering = firstView.findViewById(R.id.tv_player_buffer);

        firstView.findViewById(R.id.button4).setOnClickListener(v -> {
            if (videoPlayer.isPlaying()) {
                videoPlayer.pause();
            } else {
                videoPlayer.play();
            }
        });
        firstView.findViewById(R.id.button5).setOnClickListener(v -> videoPlayer.stop());
        firstView.findViewById(R.id.button6).setOnClickListener(v -> {
            String seekTo = ((EditText) firstView.findViewById(R.id.et_seek_time)).getText().toString();
            int time;
            try {
                time = Integer.parseInt(seekTo);
            } catch (NumberFormatException e) {
                e.printStackTrace();
                showToast("seekTo数字");
                return;
            }
            // to second
            videoPlayer.seek(time);
        });
        firstView.findViewById(R.id.button_rate).setOnClickListener(v -> {
            Button button = firstView.findViewById(R.id.button_rate);
            String text = button.getText().toString();
            switch (text) {
                case "1.0x":
                    videoPlayer.setPlayRate(1.2f);
                    button.setText("1.2x");
                    break;
                case "1.2x":
                    videoPlayer.setPlayRate(1.5f);
                    button.setText("1.5x");
                    break;
                case "1.5x":
                    videoPlayer.setPlayRate(2.0f);
                    button.setText("2.0x");
                    break;
                case "2.0x":
                    videoPlayer.setPlayRate(.5f);
                    button.setText("0.5x");
                    break;
                case "0.5x":
                    videoPlayer.setPlayRate(1.f);
                    button.setText("1.0x");
                    break;
                default:
                    break;
            }
        });
    }

    private void initDataSourceControlSection() {
        firstView.findViewById(R.id.btn_datasource).setOnClickListener(v -> {
            String vid = ((EditText) firstView.findViewById(R.id.et_vid)).getText().toString();
            String token = ((EditText) firstView.findViewById(R.id.et_token)).getText().toString();
            if (TextUtils.isEmpty(vid)) {
                showToast("vid不能为空");
                return;
            }
            if (TextUtils.isEmpty(token)) {
                showToast("token不能为空");
                return;
            }
            sp.edit().putString("vid", vid).apply();
            sp.edit().putString("token", token).apply();
            firstView.findViewById(R.id.cb_break_point_play).setEnabled(false);
            videoPlayer.setPreferredDefinitions(videoDefinitions);
//            videoPlayer.setAutoPlay(false);
            videoPlayer.setupOnlineVideoWithId(Long.parseLong(vid), token);
//            DownloadModel downloadModel = new DownloadModel();
//            downloadModel.targetFolder = "/sdcard/Android/data/com.baijiayun.videoplayer.core.demo";
//            downloadModel.targetName = "64478ec18b2990af17995903ff9fdb24_2_mp4.mp4";
//            downloadModel.definition= VideoDefinition.SD;
//            downloadModel.fileType = FileType.VIDEO;
//            videoPlayer.setupLocalVideoWithDownloadModel(downloadModel);
        });
    }

    private void initPreferredDefinitionControlSection() {
        videoDefinitions = new ArrayList<>();
        videoDefinitions.add(VideoDefinition.Audio);
        videoDefinitions.add(VideoDefinition.HD);
        videoDefinitions.add(VideoDefinition._720P);
        videoDefinitions.add(VideoDefinition.SD);
        videoDefinitions.add(VideoDefinition.SHD);
        videoDefinitions.add(VideoDefinition._1080P);

        notifyDefinitionChanged();
    }

    private void notifyDefinitionChanged() {
        StringBuilder text = new StringBuilder("preferred definitions:");
        for (VideoDefinition v : videoDefinitions) {
            text.append(" ").append(v.name());
        }
        TextView tv = firstView.findViewById(R.id.tv_definitions);
        tv.setText(text.toString());
    }

    @Override
    public void onDestroy(){
        super.onDestroy();
        videoPlayer = null;
        playerView = null;
        sp = null;
        videoDefinitions = null;

        firstView = null;
    }

    private void showToast(String toast) {
        Toast.makeText(getActivity(), toast, Toast.LENGTH_SHORT).show();
    }


}
