package com.baijiayun.video;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import androidx.annotation.Nullable;

import com.baijiahulian.common.permission.AppPermissions;
import com.baijiayun.BJYPlayerSDK;
import com.baijiayun.LauncherActivity;
import com.baijiayun.download.DownloadModel;
import com.baijiayun.download.SimpleVideoDownloadActivity;
import com.google.android.material.tabs.TabLayout;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;
import androidx.core.content.ContextCompat;
import androidx.viewpager.widget.ViewPager;
import androidx.appcompat.app.AppCompatActivity;

import android.view.WindowManager;
import android.widget.Toast;

import com.baijiayun.App;
import com.baijiayun.videoplayer.widget.BJYPlayerView;
import com.baijiayun.videoplayer.IBJYVideoPlayer;
import com.baijiayun.videoplayer.VideoPlayerFactory;
import com.baijiayun.videoplayer.core.demo.R;

import java.util.ArrayList;
import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;

/**
 * Created by Shubo on 2018/5/22.
 */
public class NewVideoActivity extends AppCompatActivity {

    private BJYPlayerView playerView;

    private IBJYVideoPlayer videoPlayer;

    private ViewPager viewPager;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_video);
        viewPager = findViewById(R.id.activity_viewPager);

        initViews();
        initPlayer();

        initViewPager();

        if(getIntent() != null){
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                DownloadModel downloadModel = (DownloadModel) getIntent().getSerializableExtra("videoDownloadInfo");
                if(downloadModel != null) {
                    videoPlayer.setupLocalVideoWithDownloadModel(downloadModel);
                }
            } else {
                AppPermissions.newPermissions(NewVideoActivity.this)
                        .request(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(aBoolean -> {
                            if (aBoolean) {
                                DownloadModel downloadModel = (DownloadModel) getIntent().getSerializableExtra("videoDownloadInfo");
                                if(downloadModel != null) {
                                    videoPlayer.setupLocalVideoWithDownloadModel(downloadModel);
                                }
                            } else {
                                Toast.makeText(NewVideoActivity.this, "没有获取读写sd卡权限", Toast.LENGTH_SHORT).show();
                            }
                        });
            }
        }
    }

    private void initViewPager(){
        List<Fragment> fragments = new ArrayList<>();

        FirstFragment firstFragment = new FirstFragment();
        SecondFragment secondFragment = new SecondFragment();

        firstFragment.setArguments(this, playerView, videoPlayer);
        secondFragment.setArguments(this, videoPlayer);

        fragments.add(firstFragment);
        fragments.add(secondFragment);

        ViewPagerAdapter adapter = new ViewPagerAdapter(this, fragments, getSupportFragmentManager());

        TabLayout mTabLayout = findViewById(R.id.tab_layout);

        mTabLayout.addTab(mTabLayout.newTab().setText("操作面板"));
        mTabLayout.addTab(mTabLayout.newTab().setText("上报统计"));

        mTabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
               viewPager.setCurrentItem(tab.getPosition());
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {
                // do nothing
            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {
                // do nothing
            }
        });

        viewPager.setAdapter(adapter);
        viewPager.setOffscreenPageLimit(2);
        viewPager.addOnPageChangeListener(new TabLayout.TabLayoutOnPageChangeListener(mTabLayout));
    }


    private void initPlayer() {
        videoPlayer = new VideoPlayerFactory.Builder()
                .setContext(this)
                //开启循环播放
                .setSupportLooping(App.isLoopPlay)
                //关闭后台音频播放
                .setSupportBackgroundAudio(App.isPlayBackground)
                //设置上报信息
                //.setUserInfo("点播回放SDK", "学员1")
                //开启记忆播放
                .setSupportBreakPointPlay(App.isMemoryPlay)
                //设置在线播放清晰度匹配规则，这里仅传Audio则优先播放纯音频，如无纯音频则播放服务器返回的默认清晰度
                //.setPreferredDefinitions(new ArrayList<VideoDefinition>(){{add(VideoDefinition.Audio);}})
                //绑定activity生命周期
                .setLifecycle(getLifecycle())
                .build();

        playerView = findViewById(R.id.activity_new_video_fl);
        videoPlayer.bindPlayerView(playerView);
        //TODO 需在bindPlayerView之后设置，否则报NPE
        //设置视频渲染载体，默认textureView,IRender.RENDER_TYPE_TEXTURE_VIEW和IRender.RENDER_TYPE_SURFACE_VIEW可选
//        playerView.setRenderType(IRender.RENDER_TYPE_TEXTURE_VIEW);
//        //设置视频画面裁剪方式，默认16：9
//        playerView.setAspectRatio(AspectRatio.AspectRatio_16_9);
    }

    private void initViews() {
//
//        playerView = VideoPlayerFactory.createPlayerView(this);
//        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
//        params.gravity = Gravity.CENTER;
//        frameLayout.addView(playerView, params);
    }

    @Override
    public void onDestroy(){
        super.onDestroy();
    }

}
